;*********************************************************************
; JoyKontrol - Analog joystick to MIDI converter                     *
;*********************************************************************
; (c) 2005 SM-Sound / Seb Francis                                    *
;*********************************************************************
;
; Main code
;
;*********************************************************************
;                                                               
;    Connections:
;
;    RA0   <- X1
;    RA1   <- Y1
;    RA2   <- Z1
;    RA3   <- X2
;    RA4   -> (open drain output)
;    RA5   <- Y2
;
;    RB0   ->
;    RB1   ->
;    RB2   ->
;    RB3   ->
;    RB4   <- EDIT button
;    RB5   <- MEMORY button
;    RB6   ->
;    RB7   ->
;
;    RC0   -> LED AN1
;    RC1   -> LED AN2
;    RC2   -> LED AN3
;    RC3   -> Debug out 1
;    RC4   -> Debug out 2
;    RC5   -> Debug out 3
;    RC6   -> MIDI Out
;    RC7   <- Used by USART
;
;    RD0   -> LED a
;    RD1   -> LED b
;    RD2   -> LED c
;    RD3   -> LED d
;    RD4   -> LED e
;    RD5   -> LED f
;    RD6   -> LED g
;    RD7   -> LED DP
;
;    RE0   <- Z2
;    RE1   <- Data Entry pot
;    RE2   <- (analog)
;                                                                  
;                                                                     
;*********************************************************************


	list      P=PIC16F877A	; List directive to define processor
	#include <P16F877A.INC>
	#include <const.inc>	; Constants + Memory map
	#include <misc.inc>	; Various macros
	

; '__CONFIG' directive is used to embed configuration data within .asm file.
; The labels following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.
	
	__CONFIG _CP_OFF & _DEBUG_OFF & _WRT_OFF & _CPD_OFF & _LVP_OFF & _BODEN_ON & _PWRTE_ON & _WDT_OFF & _HS_OSC


;**********************************************************************


	org     0x0000		; Processor reset vector

	clrf    PCLATH		; Ensure page bits are cleared
	goto    Init		; Go to beginning of program


	org     0x0004		; Interrupt vector location
	
	movwf   W_TEMP		; Save W, STATUS, PCLATH and FSR
	swapf	STATUS,w	; Use swapf, not movf so not to disturb Z flag
	movwf	STATUS_TEMP
	movfw   PCLATH
	movwf	PCLATH_TEMP
	movfw	FSR
	movwf	FSR_TEMP
	
	
	movlw	high LEDNumbers	; Set PCLATH for jump tables
	movwf	PCLATH

; Call the interupt handlers

	clrf	STATUS		; RAM bank 0, IRP 0
	btfsc	INTCON,TMR0IF
	call	UpdateLEDs
	
	clrf	STATUS		; RAM bank 0, IRP 0
	btfsc	INTCON,RBIF
	call	ButtonsChange

	clrf	STATUS		; RAM bank 0, IRP 0
	btfsc	PIR1,TMR1IF
	call	ReadButtons

	clrf	STATUS		; RAM bank 0, IRP 0
	btfsc	PIR1,RCIF
	call	MIDIByteReceived

; --
	movfw	FSR_TEMP	; Restore W, STATUS, PCLATH and FSR
	movwf	FSR
	movfw	PCLATH_TEMP
	movwf	PCLATH
	swapf	STATUS_TEMP,w	; Use swapf, not movf so not to disturb Z flag
	movwf	STATUS
	swapf	W_TEMP,f	; Use swapf, not movf so not to disturb Z flag
	swapf	W_TEMP,w	; Swap nibbles again to get them correct
InterruptEnd
	retfie			; Return from interrupt


;**********************************************************************


Init

; Bank 3 Setup
	bsf	STATUS,RP0	; Bank 3
	bsf	STATUS,RP1
	
	clrf	EECON1		; Access EEPROM data memory, EEPROM write disable
	
	
; Bank 0 Setup
	clrf	STATUS		; RAM bank 0, IRP 0
	
	clrf	PORTA		; Set all outputs to off initially
	clrf	PORTB
	clrf	PORTC
	clrf	PORTD
	clrf	PORTE
	
	movlw	b'01101000'	; Global disable interrupts, enable interrupts: peripheral, TMR0, RB change
	movwf	INTCON
	
	clrf	PIR1		; Clear all interrupt flags
	clrf	PIR2
	
	movlw	b'00010000'	; TMR1: prescale 1:2, oscillator disabled, internal clock, timer stopped
	movwf	T1CON
	
	clrf	T2CON		; TMR2: off
	clrf	SSPCON		; SSP: off
	clrf	CCP1CON		; CCP1: off
	clrf	CCP2CON		; CCP2: off
	
	movlw	b'10000000'	; USART: enabled, 8 bit reception, reception (CREN) disabled, address detection disabled
	movwf	RCSTA
	
	movlw	b'10000001'	; A/D: clock FOSC/64, select AN0, conversion not in progress, converter on
	movwf	ADCON0
	
	
; Initialise variables in bank 0

	
	; Set all current values and last values sent to 0xFF
	movlw	low LAST_X1_POS
	movwf	FSR
	movlw	CURRENT_DATA - LAST_X1_POS + 1
	movwf	TEMP1
	
	movlw	0xFF
	movwf	INDF
	incf	FSR
	decfsz	TEMP1
	goto	$-3
	
	; CC_LEARN = none, CC_VALUE_LEARN = no value
	movwf	CC_LEARN
	movwf	CC_VALUE_LEARN
	
	; Set all LED digits blank
	movwf	LED_DIGIT1
	movwf	LED_DIGIT2
	movwf	LED_DIGIT3
	
	; Clear flags
	clrf	FLAGS
	clrf	FLAGS2
	
	; Initially set LED display counter to use for on-delay
	clrf	LED_INTERRUPT_COUNT
	incf	LED_INTERRUPT_COUNT
	
	; Clear mode indicators
	clrf	TEST_MODE_SELECT
	clrf	EDIT_MODE_STAGE
	clrf	EDIT_MODE_CTRL_SELECTED
	
	
; Load settings from EEPROM

	; Read MEMORY_LOCATION
	movlw	EE_MEMORY_LOCATION
	movwf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	MEMORY_LOCATION
	
	; If MEMORY_LOCATION <> 0xFF, then EEPROM is valid
	incf	MEMORY_LOCATION,w
	jnz	EEPROM_Valid
	
	; Set default value for MEMORY_LOCATION
	movlw	1
	movwf	MEMORY_LOCATION
	
	; Write default values to EEPROM
	movwf	TEMP2
	movlw	EE_MEMORY_LOCATION
	movfw	TEMP1
	call	WriteByteToEEPROM_Int
	incf	TEMP1	; MIDI_CHANNEL
	clrf	TEMP2
	call	WriteByteToEEPROM_Int
	movlw	0x80
	movwf	TEMP2
	incf	TEMP1	; OFFSET_X1
	call	WriteByteToEEPROM_Int
	incf	TEMP1	; OFFSET_Y1
	call	WriteByteToEEPROM_Int
	incf	TEMP1	; OFFSET_Z1
	call	WriteByteToEEPROM_Int
	incf	TEMP1	; OFFSET_X2
	call	WriteByteToEEPROM_Int
	incf	TEMP1	; OFFSET_Y2
	call	WriteByteToEEPROM_Int
	incf	TEMP1	; OFFSET_Z2
	call	WriteByteToEEPROM_Int
	
	
EEPROM_Valid
	; Load MIDI_CHANNEL & OFFSET_*
	movlw	EE_MIDI_CHANNEL
	movwf	TEMP1
	call	ReadByteFromEEPROM_Int
	movwf	MIDI_CHANNEL
	
	incf	TEMP1	; OFFSET_X1
	call	ReadByteFromEEPROM_Int
	movwf	OFFSET_X1
	
	incf	TEMP1	; OFFSET_Y1
	call	ReadByteFromEEPROM_Int
	movwf	OFFSET_Y1
	
	incf	TEMP1	; OFFSET_Z1
	call	ReadByteFromEEPROM_Int
	movwf	OFFSET_Z1
	
	incf	TEMP1	; OFFSET_X2
	call	ReadByteFromEEPROM_Int
	movwf	OFFSET_X2
	
	incf	TEMP1	; OFFSET_Y2
	call	ReadByteFromEEPROM_Int
	movwf	OFFSET_Y2
	
	incf	TEMP1	; OFFSET_Z2
	call	ReadByteFromEEPROM_Int
	movwf	OFFSET_Z2
	

	; Now load current memory location
	call LoadMemoryLocation_Init
	
	
	
; Bank 1 Setup
	bsf	STATUS,RP0	; RAM bank 1
	
	movlw	b'00000110'	; PORTB pullups enabled, Internal TMR0 clock, Pre-scaler assigned to TMR0, 1:128 (interrupt every 6.5536ms)
	movwf	OPTION_REG
	
	movlw	b'00101111'	; RA0, RA1, RA2, RA3, RA5 inputs
	movwf	TRISA
	
	movlw	b'00110000'	; RB4, RB5 inputs
	movwf	TRISB
	
	movlw	b'10000000'	; RC7 USART input (not used)
	movwf	TRISC
	
	clrf	TRISD		; PORTD all outputs
	
	movlw	b'00000111'	; RE0, RE1, RE2 inputs
	movwf	TRISE
	
	movlw	b'00100001'	; Peripheral interrupts 1: USART receive on, TMR1 on, everything else off
	movwf	PIE1
	
	clrf	PIE2		; Peripheral interrupts 2: All off
	
	clrf	SSPCON2		; SSP settings
	clrf	SSPSTAT		; SSP settings
	
	movlw	b'00100000'	; USART: 8 bit transmission, transmit enabled, asynchronous mode, low baud rate
	movwf	TXSTA
	
	movlw	D'9'		; USART: Baud rate 31,250
	movwf	SPBRG
	
	clrf	CMCON		; Comparator
	
	clrf	CVRCON		; Comparator voltage reference off
	
	movlw	b'01000000'	; A/D: result left justified, clock FOSC/64, all analog inputs enabled
	movwf	ADCON1


; Bank 2 Setup
; 	bcf	STATUS,RP0	; RAM bank 2
; 	bsf	STATUS,RP1
	
	; Nothing to do in bank 2


	
;--
	clrf	STATUS		; RAM bank 0, IRP 0
	
	movlw	high LEDNumbers	; Set PCLATH for jump tables
	movwf	PCLATH

	bsf	RCSTA,CREN	; Enable MIDI receive
	bcf	INTCON,TMR0IF	; Reset TMR0 flag
	bsf	INTCON,GIE	; Enable interrupts


;#######################################################################

; RAM Bank 0 !

; ----------------------------------------------------------------------


Main
	movfw	EDIT_MODE_STAGE		; Edit mode?
	jnz	EditMode

	btfsc	FLAGS,MEMORY_PRESSED	; Memory button pressed?
	call	MemorySelect

	btfsc	FLAGS,CALIBRATE_OFFSETS	; Calibrate offsets?
	call	CalibrateOffsets
		
	movfw	TEST_MODE_SELECT	; Test mode?
	jnz	TestMode

	
	clrf	MTEMP1
Main_Loop
	; Controller id in MTEMP1
	;
	; Uses info in OFFSET_n, CTRL_MODE, CC_n_POS/NEG, MIN_n_POS/NEG, MAX_n_POS/NEG, LAST_n_POS/NEG
	; Changes MTEMP2, MTEMP3, AARGB0, AARGB1, AARGB2, BARGB0, FSR
	;
	; Reads from A/D channel and sends appropriate MIDI message(s)
	call	ADConversionToMIDI
	
	; If controller id = 6, jump to outer loop
	movlw	6
	subwf	MTEMP1,w
	jz	Main
	
	incf	MTEMP1		; Point to next controller
	goto	Main_Loop	; Loop
	

	
;-----------------------------------------------------------------------------------------

; Edit sequence (press EDIT button for each step) - MEMORY cancels edit
; ->
; 'Edt' -> Move joystick or pot to select 'H1 ', 'y1 ', 'Z1 ', 'H2 ', 'y2 ', 'Z2 ', 'DAt'
; ->
; '-S-' or '-A-' to choose between simple and advanced mode
; ->
; MIDI CC number 0-127 or 'Pch' or 'Aft' or 'PCh' or 'Off'  - at this point any incoming MIDI message on any channel sets this
; ->
; 'n.nn' lowest value 0-127  (can be higher than highest value for reverse polarity)
; ->
; 'nn.n' highest value 0-127  (can be lower than lowest value for reverse polarity)
; ->
; [advanced mode] (edit CC params for negative controller positions)
; MIDI CC number 0-127. or 'Pch.' or 'Aft.' or 'PCh.' or 'Off.'  - at this point any incoming MIDI message on any channel sets this
; ->
; 'n.nn.' lowest value 0-127  (can be higher than highest value for reverse polarity)
; ->
; 'nn.n.' highest value 0-127  (can be lower than lowest value for reverse polarity)
; ->
; [/advanced mode]
; Back to blank, settings saved in current memory location
;
;
; Press EDIT button twice
; ->
; 'Cnn' MIDI output channel 1-16
;
;
; EDIT_MODE_STAGE
; ---------------
; 0=Not in edit mode
; 1=Controller select
; 2=Set MIDI channel
; 3=Simple/Advanced mode select
; 4=CC select
; 5=Min select
; 6=Max select
; 7=CC select (NEG)
; 8=Min select (NEG)
; 9=Max select (NEG)
; 10=End of edit mode reached (save and exit)
; 11=End of MIDI channel edit mode reached (save MIDI channel and exit)
EditMode
	; Set last controller values to 0xFF
	movlw	0xFF
	movwf	EM_LAST_X1
	movwf	EM_LAST_Y1
	movwf	EM_LAST_Z1
	movwf	EM_LAST_X2
	movwf	EM_LAST_Y2
	movwf	EM_LAST_Z2
	movwf	EM_LAST_DATA
	
	
EditMode_Loop
	btfss	FLAGS2,RESET_LAST_EDIT_VALUE	; Check if LAST_EDIT_VALUE needs resetting
	goto	$+4
	movlw	0xFF			; Initialise LAST_EDIT_VALUE with 0xFF
	movwf	LAST_EDIT_VALUE
	bcf	FLAGS2,RESET_LAST_EDIT_VALUE

	movfw	EDIT_MODE_STAGE		; Copy EDIT_MODE_STAGE to MTEMP1
	movwf	MTEMP1
	
	; If stage == 0 back to main loop
	jz	Main
	
	bcf	INTCON,TMR0IE		; LED updates off
	clrf	LED_INTERRUPT_COUNT	; Keep display turned on
	incf	LED_INTERRUPT_COUNT
	bsf	INTCON,TMR0IE		; LED updates on
	
	; Jump to appropriate stage
	decf	MTEMP1
	jz	EditMode_CtrlSelect
	decf	MTEMP1
	jz	EditMode_ChanSelect
	decf	MTEMP1
	jz	EditMode_ModeSelect
	decf	MTEMP1
	jz	EditMode_CCSelect
	decf	MTEMP1
	jz	EditMode_MinSelect
	decf	MTEMP1
	jz	EditMode_MaxSelect
	decf	MTEMP1
	jz	EditMode_CCSelectNEG
	decf	MTEMP1
	jz	EditMode_MinSelectNEG
	decf	MTEMP1
	jz	EditMode_MaxSelectNEG
	decf	MTEMP1
	jz	EditMode_SaveAndExit
	goto	EditMode_SaveChanAndExit
	
;-------
	
EditMode_CtrlSelect
	movfw	EDIT_MODE_CTRL_SELECTED	; If controller has already been selected, don't change display
	jnz	EditMode_CtrlSelect_X1

	; Display 'Edt'
	movlw	low (LEDText_EDIT - LEDText_Base)
	movwf	MTEMP1
	call	WriteTextToLED
	
	; Copy MIDI_CHANNEL to EDIT_MODE_MIDI_CHANNEL
	movfw	MIDI_CHANNEL
	movwf	EDIT_MODE_MIDI_CHANNEL
	
	
TestForBigValueChange	macro	EM_LAST, NextJumpPoint
	incf	EM_LAST,w		; Is last value not 0xFF any more?
	jnz	$+4
	
	movfw	ADRESH			; Store 1st value
	movwf	EM_LAST
	goto	NextJumpPoint		; Skip the rest
	
; Value not 0xFF any more
	movfw	ADRESH			; If value hasn't changed by at least 20, loop around again
	subwf	EM_LAST,w
	movwf	MTEMP1			; Difference negative?
	btfsc	MTEMP1,7
	goto	$+5
	
; Postive
	addlw	0x100-D'20'		; < 20 ?
	jnc	NextJumpPoint		; Skip the rest
	goto	$+4
	
; Negative
	addlw	D'19'			; > -20 ?
	jc	NextJumpPoint		; Skip the rest
	
	
; Changed by at least 20
	movfw	ADRESH			; Store value
	movwf	EM_LAST
endm

	
EditMode_CtrlSelect_X1
	bcf	ADCON0,CHS2	; Select analog input AN0 (X1)
	bcf	ADCON0,CHS1
	bcf	ADCON0,CHS0
	call	ADConversion	; Perform A/D conversion
	
	TestForBigValueChange	EM_LAST_X1, EditMode_CtrlSelect_Y1
	
	movlw	1			; Select controller
	goto	EditMode_CtrlSelected
		

EditMode_CtrlSelect_Y1
	bsf	ADCON0,CHS0	; Select analog input AN1 (Y1)
	call	ADConversion	; Perform A/D conversion
	
	TestForBigValueChange	EM_LAST_Y1, EditMode_CtrlSelect_Z1
	
	movlw	2			; Select controller
	goto	EditMode_CtrlSelected
		

EditMode_CtrlSelect_Z1
	bsf	ADCON0,CHS1	; Select analog input AN2 (Z1)
	bcf	ADCON0,CHS0
	call	ADConversion	; Perform A/D conversion
	
	TestForBigValueChange	EM_LAST_Z1, EditMode_CtrlSelect_X2
	
	movlw	3			; Select controller
	goto	EditMode_CtrlSelected
		

EditMode_CtrlSelect_X2
	bsf	ADCON0,CHS0	; Select analog input AN3 (X2)
	call	ADConversion	; Perform A/D conversion
	
	TestForBigValueChange	EM_LAST_X2, EditMode_CtrlSelect_Y2
	
	movlw	4			; Select controller
	goto	EditMode_CtrlSelected
		

EditMode_CtrlSelect_Y2
	bsf	ADCON0,CHS2	; Select analog input AN4 (Y2)
	bcf	ADCON0,CHS1
	bcf	ADCON0,CHS0
	call	ADConversion	; Perform A/D conversion
	
	TestForBigValueChange	EM_LAST_Y2, EditMode_CtrlSelect_Z2
	
	movlw	5			; Select controller
	goto	EditMode_CtrlSelected
		

EditMode_CtrlSelect_Z2
	bsf	ADCON0,CHS0	; Select analog input AN5 (Z2)
	call	ADConversion	; Perform A/D conversion
	
	TestForBigValueChange	EM_LAST_Z2, EditMode_CtrlSelect_DATA
	
	movlw	6			; Select controller
	goto	EditMode_CtrlSelected
		

EditMode_CtrlSelect_DATA
	bsf	ADCON0,CHS1	; Select analog input AN6 (DATA)
	bcf	ADCON0,CHS0
	call	ADConversion	; Perform A/D conversion
	
	TestForBigValueChange	EM_LAST_DATA, EditMode_Loop
	
	movlw	7			; Select controller
	

EditMode_CtrlSelected
	movwf	EDIT_MODE_CTRL_SELECTED
	
	; Display controller name
	movlw	low (LEDText_X1 - LEDText_Base - 3)
	addwf	EDIT_MODE_CTRL_SELECTED,w
	addwf	EDIT_MODE_CTRL_SELECTED,w
	addwf	EDIT_MODE_CTRL_SELECTED,w
	movwf	MTEMP1
	call	WriteTextToLED
	
	; Load initial Mode, CC, MIN & MAX settings
	
	; Check if simple mode or advanced mode
	movlw	b'00000001'
	movwf	MTEMP1
	bcf	STATUS,C
	
	decf	EDIT_MODE_CTRL_SELECTED,w	; Get bitmask corresponding to controller
	jz	$+5
	movwf	MTEMP2
	rlf	MTEMP1
	decfsz	MTEMP2
	goto	$-2
	
	movfw	MTEMP1
	bsf	STATUS,RP0	; RAM bank 1
	andwf	CTRL_MODE,w
	bcf	STATUS,RP0	; RAM bank 0
	
	movlw	0
	sz		; If non-zero result, then advanced mode (dual CC)
	movlw	1
	movwf	EDIT_MODE_CTRL_MODE
	
	
	movlw	low CC_X1_POS - 1
	addwf	EDIT_MODE_CTRL_SELECTED,w
	movwf	FSR
	movfw	INDF
	movwf	EDIT_MODE_CC_SELECTED_POS
	
	movlw	low CC_X1_NEG - 1
	addwf	EDIT_MODE_CTRL_SELECTED,w
	movwf	FSR
	movfw	INDF
	movwf	EDIT_MODE_CC_SELECTED_NEG
	
	movlw	low MIN_X1_POS - 1
	addwf	EDIT_MODE_CTRL_SELECTED,w
	movwf	FSR
	movfw	INDF
	movwf	EDIT_MODE_MIN_SELECTED_POS
	
	movlw	low MIN_X1_NEG - 1
	addwf	EDIT_MODE_CTRL_SELECTED,w
	movwf	FSR
	movfw	INDF
	movwf	EDIT_MODE_MIN_SELECTED_NEG
	
	movlw	low MAX_X1_POS - 1
	addwf	EDIT_MODE_CTRL_SELECTED,w
	movwf	FSR
	movfw	INDF
	movwf	EDIT_MODE_MAX_SELECTED_POS
	
	movlw	low MAX_X1_NEG - 1
	addwf	EDIT_MODE_CTRL_SELECTED,w
	movwf	FSR
	movfw	INDF
	movwf	EDIT_MODE_MAX_SELECTED_NEG
	
	goto	EditMode_Loop
	
;-------

EditMode_ChanSelect

	; Display 'Cnn'
	
	; Write 'C'
	movlw	low (LEDText_CAL - LEDText_Base)
	call	GetLEDNumber
	movwf	LED_DIGIT1
	
	; Convert EDIT_MODE_MIDI_CHANNEL + 1 to BCD
	movfw	EDIT_MODE_MIDI_CHANNEL
	movwf	AARGB1
	incf	AARGB1,w
	call	ConvertToBcd
	
	; Write to LED
	movfw	AARGB1
	call	GetLEDNumber
	snz			; If 0, digit off
	movlw	0xFF
	movwf	LED_DIGIT2
	
	movfw	AARGB2
	call	GetLEDNumber
	movwf	LED_DIGIT3
	
	
	clrf	MTEMP1
	movlw	D'15'
	movwf	MTEMP2
	; MTEMP1 - Min data value
	; MTEMP2 - Max data value
	; Returns data value in AARGB1
	call	GetDataPotValue
	
	
	incf	LAST_EDIT_VALUE,w	; Is LAST_EDIT_VALUE not 0xFF any more?
	jnz	ChanSelect_NotFirstValue
	
	movfw	AARGB1			; Store 1st value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	goto	EditMode_Loop		; Loop
	
ChanSelect_NotFirstValue
	movfw	AARGB1			; If value hasn't changed, loop around again
	subwf	LAST_EDIT_VALUE,w
	jz	EditMode_Loop
	
	movfw	AARGB1			; Store value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	
	
	movwf	EDIT_MODE_MIDI_CHANNEL	; Store value in EDIT_MODE_MIDI_CHANNEL
	
	goto	EditMode_Loop
	
;-------

EditMode_ModeSelect

	; Display '-S-' or '-A-'
	movfw	EDIT_MODE_CTRL_MODE
	movlw	low (LEDText_Simple - LEDText_Base)
	sz
	movlw	low (LEDText_Advanced - LEDText_Base)
	movwf	MTEMP1
	call	WriteTextToLED
	
	
	clrf	MTEMP1
	movlw	D'15'
	movwf	MTEMP2
	; MTEMP1 - Min data value
	; MTEMP2 - Max data value
	; Returns data value in AARGB1
	call	GetDataPotValue
	
	
	incf	LAST_EDIT_VALUE,w	; Is LAST_EDIT_VALUE not 0xFF any more?
	jnz	ModeSelect_NotFirstValue
	
	movfw	AARGB1			; Store 1st value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	goto	EditMode_Loop		; Loop
	
ModeSelect_NotFirstValue
	movfw	AARGB1			; If value hasn't changed, loop around again
	subwf	LAST_EDIT_VALUE,w
	jz	EditMode_Loop
	
	movfw	AARGB1			; Store value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	
	
	addlw	0xFF-7			; If value <= 7, EDIT_MODE_CTRL_MODE = 0, else EDIT_MODE_CTRL_MODE = 1
	movlw	0
	snc
	movlw	1
	movwf	EDIT_MODE_CTRL_MODE
	
	goto	EditMode_Loop
	
;-------

EditMode_CCSelect
	; Display current value
	
	; Pitch bend?
	movlw	D'128'
	subwf	EDIT_MODE_CC_SELECTED_POS,w
	jz	EditMode_CCSelect_PitchBend
	
	; Aftertouch?
	movlw	D'129'
	subwf	EDIT_MODE_CC_SELECTED_POS,w
	jz	EditMode_CCSelect_Aftertouch
	
	; Program change?
	movlw	D'130'
	subwf	EDIT_MODE_CC_SELECTED_POS,w
	jz	EditMode_CCSelect_ProgChange
	
	; Off?
	movlw	D'131'
	subwf	EDIT_MODE_CC_SELECTED_POS,w
	jz	EditMode_CCSelect_Off
	
; CC
	movfw	EDIT_MODE_CC_SELECTED_POS
	movwf	AARGB1
	; Call with number in AARGB1
	; Writes number to LCD
	call	DisplayNumber
	
	goto	EditMode_CCSelect_LearnMIDI


EditMode_CCSelect_PitchBend
	; Display 'Pch'
	movlw	low (LEDText_PitchBend - LEDText_Base)
	movwf	MTEMP1
	call	WriteTextToLED
	
	goto	EditMode_CCSelect_LearnMIDI
	

EditMode_CCSelect_Aftertouch
	; Display 'Aft'
	movlw	low (LEDText_Aftertouch - LEDText_Base)
	movwf	MTEMP1
	call	WriteTextToLED
	
	goto	EditMode_CCSelect_LearnMIDI
	

EditMode_CCSelect_ProgChange
	; Display 'PCh'
	movlw	low (LEDText_ProgChange - LEDText_Base)
	movwf	MTEMP1
	call	WriteTextToLED
	
	goto	EditMode_CCSelect_LearnMIDI
	
	
EditMode_CCSelect_Off
	; Display 'Off'
	movlw	low (LEDText_Off - LEDText_Base)
	movwf	MTEMP1
	call	WriteTextToLED
	

EditMode_CCSelect_LearnMIDI
	incf	CC_LEARN,w		; Learn received MIDI message?
	jz	EditMode_CCSelect_ReadPot
	
	movfw	CC_LEARN		; Copy CC_LEARN to EDIT_MODE_CC_SELECTED_POS
	movwf	EDIT_MODE_CC_SELECTED_POS
	movlw	0xFF			; Reset CC_LEARN
	movwf	CC_LEARN
	

EditMode_CCSelect_ReadPot
	clrf	MTEMP1
	movlw	D'131'
	movwf	MTEMP2
	; MTEMP1 - Min data value
	; MTEMP2 - Max data value
	; Returns data value in AARGB1
	call	GetDataPotValue
	
	
	incf	LAST_EDIT_VALUE,w	; Is LAST_EDIT_VALUE not 0xFF any more?
	jnz	CCSelect_NotFirstValue
	
	movfw	AARGB1			; Store 1st value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	goto	EditMode_Loop		; Loop
	
CCSelect_NotFirstValue
	movfw	AARGB1			; If value hasn't changed, loop around again
	subwf	LAST_EDIT_VALUE,w
	jz	EditMode_Loop
	
	movfw	AARGB1			; Store last value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	
	
	movwf	EDIT_MODE_CC_SELECTED_POS	; Store value in EDIT_MODE_CC_SELECTED_POS
	
	goto	EditMode_Loop
	
;-------

EditMode_MinSelect

	; Display value
	
	movfw	EDIT_MODE_MIN_SELECTED_POS
	movwf	AARGB1
	bcf	INTCON,TMR0IE		; LED updates off
	; Call with number in AARGB1
	; Writes number to LCD
	call	DisplayNumber
	
	bcf	LED_DIGIT1,7		; Set lefthand '.' on LED
	bsf	INTCON,TMR0IE		; LED updates on
	
	
	; Check for learned CC value
	incf	CC_VALUE_LEARN,w	; Learn received MIDI value?
	jz	EditMode_MinSelect_ReadPot
	
	movfw	CC_VALUE_LEARN		; Copy CC_VALUE_LEARN to EDIT_MODE_MIN_SELECTED_POS
	movwf	EDIT_MODE_MIN_SELECTED_POS
	movlw	0xFF			; Reset CC_VALUE_LEARN
	movwf	CC_VALUE_LEARN
	

EditMode_MinSelect_ReadPot
	clrf	MTEMP1
	movlw	D'127'
	movwf	MTEMP2
	; MTEMP1 - Min data value
	; MTEMP2 - Max data value
	; Returns data value in AARGB1
	call	GetDataPotValue
	
	
	incf	LAST_EDIT_VALUE,w	; Is LAST_EDIT_VALUE not 0xFF any more?
	jnz	MinSelect_NotFirstValue
	
	movfw	AARGB1			; Store 1st value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	goto	EditMode_Loop		; Loop
	
MinSelect_NotFirstValue
	movfw	AARGB1			; If value hasn't changed, loop around again
	subwf	LAST_EDIT_VALUE,w
	jz	EditMode_Loop
	
	movfw	AARGB1			; Store value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	
	
	movwf	EDIT_MODE_MIN_SELECTED_POS	; Store value in EDIT_MODE_MIN_SELECTED_POS
	
	goto	EditMode_Loop
	
;-------

EditMode_MaxSelect

	; Display value
	
	movfw	EDIT_MODE_MAX_SELECTED_POS
	movwf	AARGB1
	bcf	INTCON,TMR0IE		; LED updates off
	; Call with number in AARGB1
	; Writes number to LCD
	call	DisplayNumber
	
	bcf	LED_DIGIT2,7		; Set middle '.' on LED
	bsf	INTCON,TMR0IE		; LED updates on
	
	
	; Check for learned CC value
	incf	CC_VALUE_LEARN,w	; Learn received MIDI value?
	jz	EditMode_MaxSelect_ReadPot
	
	movfw	CC_VALUE_LEARN		; Copy CC_VALUE_LEARN to EDIT_MODE_MAX_SELECTED_POS
	movwf	EDIT_MODE_MAX_SELECTED_POS
	movlw	0xFF			; Reset CC_VALUE_LEARN
	movwf	CC_VALUE_LEARN
	

EditMode_MaxSelect_ReadPot
	clrf	MTEMP1
	movlw	D'127'
	movwf	MTEMP2
	; MTEMP1 - Min data value
	; MTEMP2 - Max data value
	; Returns data value in AARGB1
	call	GetDataPotValue
	
	
	incf	LAST_EDIT_VALUE,w	; Is LAST_EDIT_VALUE not 0xFF any more?
	jnz	MaxSelect_NotFirstValue
	
	movfw	AARGB1			; Store 1st value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	goto	EditMode_Loop		; Loop
	
MaxSelect_NotFirstValue
	movfw	AARGB1			; If value hasn't changed, loop around again
	subwf	LAST_EDIT_VALUE,w
	jz	EditMode_Loop
	
	movfw	AARGB1			; Store value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	
	
	movwf	EDIT_MODE_MAX_SELECTED_POS	; Store value in EDIT_MODE_MAX_SELECTED_POS
	
	goto	EditMode_Loop
	
;-------

EditMode_CCSelectNEG
	; Display current value
	
	bcf	INTCON,TMR0IE		; LED updates off
	
	; Pitch bend?
	movlw	D'128'
	subwf	EDIT_MODE_CC_SELECTED_NEG,w
	jz	EditMode_CCSelectNEG_PitchBend
	
	; Aftertouch?
	movlw	D'129'
	subwf	EDIT_MODE_CC_SELECTED_NEG,w
	jz	EditMode_CCSelectNEG_Aftertouch
	
	; Program change?
	movlw	D'130'
	subwf	EDIT_MODE_CC_SELECTED_NEG,w
	jz	EditMode_CCSelectNEG_ProgChange
	
	; Off?
	movlw	D'131'
	subwf	EDIT_MODE_CC_SELECTED_NEG,w
	jz	EditMode_CCSelectNEG_Off
	
; CC
	movfw	EDIT_MODE_CC_SELECTED_NEG
	movwf	AARGB1
	; Call with number in AARGB1
	; Writes number to LCD
	call	DisplayNumber
	
	goto	EditMode_CCSelectNEG_LearnMIDI


EditMode_CCSelectNEG_PitchBend
	; Display 'Pch'
	movlw	low (LEDText_PitchBend - LEDText_Base)
	movwf	MTEMP1
	call	WriteTextToLED
	
	goto	EditMode_CCSelectNEG_LearnMIDI
	

EditMode_CCSelectNEG_Aftertouch
	; Display 'Aft'
	movlw	low (LEDText_Aftertouch - LEDText_Base)
	movwf	MTEMP1
	call	WriteTextToLED
	
	goto	EditMode_CCSelectNEG_LearnMIDI
	

EditMode_CCSelectNEG_ProgChange
	; Display 'PCh'
	movlw	low (LEDText_ProgChange - LEDText_Base)
	movwf	MTEMP1
	call	WriteTextToLED
	
	goto	EditMode_CCSelectNEG_LearnMIDI
	
	
EditMode_CCSelectNEG_Off
	; Display 'Off'
	movlw	low (LEDText_Off - LEDText_Base)
	movwf	MTEMP1
	call	WriteTextToLED
	

EditMode_CCSelectNEG_LearnMIDI
	bcf	LED_DIGIT3,7		; Set righthand '.' on LED
	bsf	INTCON,TMR0IE		; LED updates on
	

	incf	CC_LEARN,w		; Learn received MIDI message?
	jz	EditMode_CCSelectNEG_ReadPot
	
	movfw	CC_LEARN		; Copy CC_LEARN to EDIT_MODE_CC_SELECTED_NEG
	movwf	EDIT_MODE_CC_SELECTED_NEG
	movlw	0xFF			; Reset CC_LEARN
	movwf	CC_LEARN
	

EditMode_CCSelectNEG_ReadPot
	clrf	MTEMP1
	movlw	D'131'
	movwf	MTEMP2
	; MTEMP1 - Min data value
	; MTEMP2 - Max data value
	; Returns data value in AARGB1
	call	GetDataPotValue
	
	
	incf	LAST_EDIT_VALUE,w	; Is LAST_EDIT_VALUE not 0xFF any more?
	jnz	CCSelectNEG_NotFirstValue
	
	movfw	AARGB1			; Store 1st value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	goto	EditMode_Loop		; Loop
	
CCSelectNEG_NotFirstValue
	movfw	AARGB1			; If value hasn't changed, loop around again
	subwf	LAST_EDIT_VALUE,w
	jz	EditMode_Loop
	
	movfw	AARGB1			; Store last value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	
	
	movwf	EDIT_MODE_CC_SELECTED_NEG	; Store value in EDIT_MODE_CC_SELECTED_POS
	
	
	goto	EditMode_Loop
	
;-------

EditMode_MinSelectNEG

	; Display value
	
	movfw	EDIT_MODE_MIN_SELECTED_NEG
	movwf	AARGB1
	bcf	INTCON,TMR0IE		; LED updates off
	; Call with number in AARGB1
	; Writes number to LCD
	call	DisplayNumber
	
	bcf	LED_DIGIT1,7		; Set lefthand '.' on LED
	bcf	LED_DIGIT3,7		; Set righthand '.' on LED
	bsf	INTCON,TMR0IE		; LED updates on
	
	
	; Check for learned CC value
	incf	CC_VALUE_LEARN,w	; Learn received MIDI value?
	jz	EditMode_MinSelectNEG_ReadPot
	
	movfw	CC_VALUE_LEARN		; Copy CC_VALUE_LEARN to EDIT_MODE_MIN_SELECTED_NEG
	movwf	EDIT_MODE_MIN_SELECTED_NEG
	movlw	0xFF			; Reset CC_VALUE_LEARN
	movwf	CC_VALUE_LEARN
	

EditMode_MinSelectNEG_ReadPot
	clrf	MTEMP1
	movlw	D'127'
	movwf	MTEMP2
	; MTEMP1 - Min data value
	; MTEMP2 - Max data value
	; Returns data value in AARGB1
	call	GetDataPotValue
	
	
	incf	LAST_EDIT_VALUE,w	; Is LAST_EDIT_VALUE not 0xFF any more?
	jnz	MinSelectNEG_NotFirstValue
	
	movfw	AARGB1			; Store 1st value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	goto	EditMode_Loop		; Loop
	
MinSelectNEG_NotFirstValue
	movfw	AARGB1			; If value hasn't changed, loop around again
	subwf	LAST_EDIT_VALUE,w
	jz	EditMode_Loop
	
	movfw	AARGB1			; Store value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	
	
	movwf	EDIT_MODE_MIN_SELECTED_NEG	; Store value in EDIT_MODE_MIN_SELECTED_NEG
	
	goto	EditMode_Loop
	
;-------

EditMode_MaxSelectNEG

	; Display value
	
	movfw	EDIT_MODE_MAX_SELECTED_NEG
	movwf	AARGB1
	bcf	INTCON,TMR0IE		; LED updates off
	; Call with number in AARGB1
	; Writes number to LCD
	call	DisplayNumber
	
	bcf	LED_DIGIT2,7		; Set middle '.' on LED
	bcf	LED_DIGIT3,7		; Set righthand '.' on LED
	bsf	INTCON,TMR0IE		; LED updates on
	
	
	; Check for learned CC value
	incf	CC_VALUE_LEARN,w	; Learn received MIDI value?
	jz	EditMode_MaxSelectNEG_ReadPot
	
	movfw	CC_VALUE_LEARN		; Copy CC_VALUE_LEARN to EDIT_MODE_MAX_SELECTED_NEG
	movwf	EDIT_MODE_MAX_SELECTED_NEG
	movlw	0xFF			; Reset CC_VALUE_LEARN
	movwf	CC_VALUE_LEARN
	

EditMode_MaxSelectNEG_ReadPot
	clrf	MTEMP1
	movlw	D'127'
	movwf	MTEMP2
	; MTEMP1 - Min data value
	; MTEMP2 - Max data value
	; Returns data value in AARGB1
	call	GetDataPotValue
	
	
	incf	LAST_EDIT_VALUE,w	; Is LAST_EDIT_VALUE not 0xFF any more?
	jnz	MaxSelectNEG_NotFirstValue
	
	movfw	AARGB1			; Store 1st value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	goto	EditMode_Loop		; Loop
	
MaxSelectNEG_NotFirstValue
	movfw	AARGB1			; If value hasn't changed, loop around again
	subwf	LAST_EDIT_VALUE,w
	jz	EditMode_Loop
	
	movfw	AARGB1			; Store value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	
	
	movwf	EDIT_MODE_MAX_SELECTED_NEG	; Store value in EDIT_MODE_MAX_SELECTED_NEG
	
	goto	EditMode_Loop
	
;-------

EditMode_SaveAndExit

	; MEMORY_LOCATION * EE_MEM_WIDTH - EE_MEM1_START -> AARGB0, AARGB1
	movlw	EE_MEM_WIDTH
	movwf	AARGB0
	movfw	MEMORY_LOCATION
	movwf	BARGB0
	bsf	PCLATH,3
	call	FXM0808U
	bcf	PCLATH,3

	movlw	EE_MEM1_START
	subwf	AARGB1,f
	
	; EEPROM memory location start address in AARGB1
	
	movfw	EDIT_MODE_CTRL_MODE
	jnz	EditMode_SaveAndExit_AdvMode
	
; Std mode
	movlw	b'11111110'
	movwf	MTEMP2
	bsf	STATUS,C
	
	decf	EDIT_MODE_CTRL_SELECTED,w
	jz	$+5
	movwf	MTEMP1
	rlf	MTEMP2
	decfsz	MTEMP1
	goto	$-2
	
	; Mask out bit
	movfw	MTEMP2
	bsf	STATUS,RP0	; RAM bank 1
	andwf	CTRL_MODE
	bcf	STATUS,RP0	; RAM bank 0
	
	goto	EditMode_SaveAndExit_SaveMode
	
EditMode_SaveAndExit_AdvMode
	movlw	b'00000001'
	movwf	MTEMP2
	bcf	STATUS,C
	
	decf	EDIT_MODE_CTRL_SELECTED,w
	jz	$+5
	movwf	MTEMP1
	rlf	MTEMP2
	decfsz	MTEMP1
	goto	$-2
	
	; Or in bit
	movfw	MTEMP2
	bsf	STATUS,RP0	; RAM bank 1
	iorwf	CTRL_MODE
	bcf	STATUS,RP0	; RAM bank 0
	
EditMode_SaveAndExit_SaveMode
	; Write CTRL_MODE to EE_CTRL_MODE in EEPROM
	movfw	AARGB1
	movwf	MTEMP1
	bsf	STATUS,RP0	; RAM bank 1
	movfw	CTRL_MODE
	bcf	STATUS,RP0	; RAM bank 0
	movwf	MTEMP2
	call	WriteByteToEEPROM_Main


	movfw	EDIT_MODE_CTRL_SELECTED	; Point MTEMP1 to EE_CC_n_POS
	addwf	AARGB1,w
	movwf	MTEMP1

	; Point FSR to CC_n_POS
	movfw	EDIT_MODE_CTRL_SELECTED
	addlw	low CC_X1_POS - 1
	movwf	FSR

	; Copy data to main variables & Save to EEPROM
	movfw	EDIT_MODE_CC_SELECTED_POS
	movwf	INDF
	movwf	MTEMP2
	call	WriteByteToEEPROM_Main
	
	movlw	7		; Point FSR & MTEMP1 to CC_n_NEG & EE_CC_n_NEG
	addwf	FSR
	addwf	MTEMP1
	movfw	EDIT_MODE_CC_SELECTED_NEG
	movwf	INDF
	movwf	MTEMP2
	call	WriteByteToEEPROM_Main
	
	movlw	7		; Point FSR & MTEMP1 to MIN_n_POS & EE_MIN_n_POS
	addwf	FSR
	addwf	MTEMP1
	movfw	EDIT_MODE_MIN_SELECTED_POS
	movwf	INDF
	movwf	MTEMP2
	call	WriteByteToEEPROM_Main
	
	movlw	7		; Point FSR & MTEMP1 to MIN_n_NEG & EE_MIN_n_NEG
	addwf	FSR
	addwf	MTEMP1
	movfw	EDIT_MODE_MIN_SELECTED_NEG
	movwf	INDF
	movwf	MTEMP2
	call	WriteByteToEEPROM_Main
	
	movlw	7		; Point FSR & MTEMP1 to MAX_n_POS & EE_MAX_n_POS
	addwf	FSR
	addwf	MTEMP1
	movfw	EDIT_MODE_MAX_SELECTED_POS
	movwf	INDF
	movwf	MTEMP2
	call	WriteByteToEEPROM_Main
	
	movlw	7		; Point FSR & MTEMP1 to MAX_n_NEG & EE_MAX_n_NEG
	addwf	FSR
	addwf	MTEMP1
	movfw	EDIT_MODE_MAX_SELECTED_NEG
	movwf	INDF
	movwf	MTEMP2
	call	WriteByteToEEPROM_Main
	
	
	; Exit edit mode
	clrf	EDIT_MODE_CTRL_SELECTED
	clrf	EDIT_MODE_STAGE
	
	bcf	INTCON,TMR0IE		; LED updates off
	clrf	LED_INTERRUPT_COUNT	; Turn off display
	decf	LED_INTERRUPT_COUNT
	bsf	INTCON,TMR0IE		; LED updates on
	
	
	bsf	FLAGS2,SUPPRESS_DATA_KNOB_CHANGE	; Make sure data knob doesn't trigger change even if in new position
	goto	Main
	
;-------

EditMode_SaveChanAndExit
	; Save EDIT_MODE_MIDI_CHANNEL to MIDI_CHANNEL and EEPROM
	movlw	EE_MIDI_CHANNEL
	movwf	MTEMP1
	movfw	EDIT_MODE_MIDI_CHANNEL
	movwf	MIDI_CHANNEL
	movwf	MTEMP2
	call	WriteByteToEEPROM_Main
	
	
	; Exit edit mode
	clrf	EDIT_MODE_STAGE
	
	bcf	INTCON,TMR0IE		; LED updates off
	clrf	LED_INTERRUPT_COUNT	; Turn off display
	decf	LED_INTERRUPT_COUNT
	bsf	INTCON,TMR0IE		; LED updates on
	
	bsf	FLAGS2,SUPPRESS_DATA_KNOB_CHANGE	; Make sure data knob doesn't trigger change even if in new position
	goto	Main
	
	
;-----------------------------------------------------------------------------------------


; Hold MEMORY and turn DATA to select memory location (display shows 0 to 9)
; Turn DATA beyond 9 to enter test modes: 'H1 ', 'y1 ', 'Z1 ', 'H2 ', 'y2 ', 'Z2 ', 'DAt'
MemorySelect
	movlw	0xFF		; Initialise LAST_EDIT_VALUE with 0xFF
	movwf	LAST_EDIT_VALUE
	
	
	; First display what is currently selected
	
	movfw	TEST_MODE_SELECT		; Test mode?
	jnz	MemorySelect_DisplayTestMode
	
	
	; Display memory location number
	movlw	0xFF
	movwf	LED_DIGIT1
	movwf	LED_DIGIT2
	movfw	MEMORY_LOCATION
	call	GetLEDNumber
	movwf	LED_DIGIT3
	
	goto	MemorySelect_Loop
	
	
MemorySelect_DisplayTestMode
	; Display controller name
	movlw	low (LEDText_X1 - LEDText_Base - 3)
	addwf	TEST_MODE_SELECT,w
	addwf	TEST_MODE_SELECT,w
	addwf	TEST_MODE_SELECT,w
	movwf	MTEMP1
	call	WriteTextToLED
	
	
MemorySelect_Loop
	btfss	FLAGS,MEMORY_PRESSED	; Memory button released?
	goto	LoadMemoryLocation	; Load memory and return
	
	
	bcf	INTCON,TMR0IE		; LED updates off
	clrf	LED_INTERRUPT_COUNT	; Keep display turned on
	incf	LED_INTERRUPT_COUNT
	bsf	INTCON,TMR0IE		; LED updates on
	
	movlw	D'1'
	movwf	MTEMP1
	movlw	D'12'
	movwf	MTEMP2
	; MTEMP1 - Min data value
	; MTEMP2 - Max data value
	; Returns data value in AARGB1
	call	GetDataPotValue
	
	
	incf	LAST_EDIT_VALUE,w	; Is LAST_EDIT_VALUE not 0xFF any more?
	jnz	MemorySelect_NotFirstValue
	
	movfw	AARGB1			; Store 1st value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	goto	MemorySelect_Loop	; Loop
	
MemorySelect_NotFirstValue
	movfw	AARGB1			; If value hasn't changed, loop around again
	subwf	LAST_EDIT_VALUE,w
	jz	MemorySelect_Loop
	
	movfw	AARGB1			; Store value in LAST_EDIT_VALUE
	movwf	LAST_EDIT_VALUE
	
	movlw	0xFF-D'5'		; AARGB1 > 5 ?
	addwf	AARGB1,w
	jc	MemorySelect_TestMode
	
	clrf	TEST_MODE_SELECT
	
	movfw	AARGB1			; Select memory location
	movwf	MEMORY_LOCATION
	
	; Display memory location number
	call	GetLEDNumber
	movwf	LED_DIGIT3
	movlw	0xFF
	movwf	LED_DIGIT1
	movwf	LED_DIGIT2
	
	goto	MemorySelect_Loop	; Loop
	
	
MemorySelect_TestMode
	addlw	1			; Select controller to be tested
	movwf	TEST_MODE_SELECT
	
	; Display controller name
	movlw	low (LEDText_X1 - LEDText_Base - 3)
	addwf	TEST_MODE_SELECT,w
	addwf	TEST_MODE_SELECT,w
	addwf	TEST_MODE_SELECT,w
	movwf	MTEMP1
	call	WriteTextToLED
	
	goto	MemorySelect_Loop	; Loop
	
	
;-----------------------------------------------------------------------------------------
	
	
CalibrateOffsets
	movlw	EE_OFFSET_X1	; Base address in EEPROM to store offsets
	movwf	MTEMP1
	

	bcf	ADCON0,CHS2	; Select analog input AN0 (X1)
	bcf	ADCON0,CHS1
	bcf	ADCON0,CHS0
	call	ADConversionRaw	; Perform A/D conversion (with no logic to stop flickering)
	
	movfw	ADRESH		; Save offset
	movwf	OFFSET_X1
	movwf	MTEMP2		; Store in EEPROM
	call	WriteByteToEEPROM_Main
	
	
	bsf	ADCON0,CHS0	; Select analog input AN1 (Y1)
	call	ADConversionRaw	; Perform A/D conversion (with no logic to stop flickering)
	
	movfw	ADRESH		; Save offset
	movwf	OFFSET_Y1
	movwf	MTEMP2		; Store in EEPROM
	call	WriteByteToEEPROM_Main
	
	
	bsf	ADCON0,CHS1	; Select analog input AN2 (Z1)
	bcf	ADCON0,CHS0
	call	ADConversionRaw	; Perform A/D conversion (with no logic to stop flickering)
	
	movfw	ADRESH		; Save offset
	movwf	OFFSET_Z1
	movwf	MTEMP2		; Store in EEPROM
	call	WriteByteToEEPROM_Main
	
	
	bsf	ADCON0,CHS0	; Select analog input AN3 (X2)
	call	ADConversionRaw	; Perform A/D conversion (with no logic to stop flickering)
	
	movfw	ADRESH		; Save offset
	movwf	OFFSET_X2
	movwf	MTEMP2		; Store in EEPROM
	call	WriteByteToEEPROM_Main
	
	
	bsf	ADCON0,CHS2	; Select analog input AN4 (Y2)
	bcf	ADCON0,CHS1
	bcf	ADCON0,CHS0
	call	ADConversionRaw	; Perform A/D conversion (with no logic to stop flickering)
	
	movfw	ADRESH		; Save offset
	movwf	OFFSET_Y2
	movwf	MTEMP2		; Store in EEPROM
	call	WriteByteToEEPROM_Main
	
	
	bsf	ADCON0,CHS0	; Select analog input AN5 (Z2)
	call	ADConversionRaw	; Perform A/D conversion (with no logic to stop flickering)
	
	movfw	ADRESH		; Save offset
	movwf	OFFSET_Z2
	movwf	MTEMP2		; Store in EEPROM
	call	WriteByteToEEPROM_Main
	
	
	; Display 'CAL' on LED
	movlw	low (LEDText_CAL - LEDText_Base)
	movwf	MTEMP1
	call	WriteTextToLED
	
	bcf	INTCON,TMR0IE		; LED updates off
	clrf	LED_INTERRUPT_COUNT	; Turn on display for 1.7s
	incf	LED_INTERRUPT_COUNT
	bsf	INTCON,TMR0IE		; LED updates on
	
	incfsz	LED_INTERRUPT_COUNT,w	; Wait until 'CAL' has been visible for 1.7s
	goto	$-1
	
	bcf	FLAGS,CALIBRATE_OFFSETS	; Clear calibrate offsets flag
	return
	
	
;-----------------------------------------------------------------------------------------


TestMode
	movfw	TEST_MODE_SELECT	; Copy controller number to MTEMP3
	movwf	MTEMP3
	
	; Shift it to match 3 analog input select bits
	decf	MTEMP3
	rlf	MTEMP3
	rlf	MTEMP3
	rlf	MTEMP3
	movlw	b'00111000'
	andwf	MTEMP3,w
	
	bcf	ADCON0,CHS2	; Clear analog input select bits
	bcf	ADCON0,CHS1
	bcf	ADCON0,CHS0
	
	iorwf	ADCON0		; Select analog input

	call	ADConversion	; Perform A/D conversion
	
	movfw	ADRESH		; Copy result to AARGB1
	movwf	AARGB1
	
	; Call with number in AARGB1
	; Writes number to LCD
	call	DisplayNumber
	
	goto	Main		; Loop
	
	
;-----------------------------------------------------------------------------------------


AntiTwitchWidth		EQU	0x2

; Perform A/D conversion
; Takes about 25us in total
;
; Changes: FSR
ADConversion
	wait_us_M D'20'		; Wait 20us for A/D acquisition
	
	bsf	ADCON0,GO_DONE	; Start converstion
	btfsc	ADCON0,GO_DONE	; Loop until converstion complete
	goto	$-1
	
	movfw	ADRESH		; Invert A/D result
	sublw	0xFF
	movwf	ADRESH
	
	; Point to current A/D channel value
	movlw	b'00111000'
	andwf	ADCON0,w
	movwf	FSR
	bcf	STATUS,C
	rrf	FSR
	rrf	FSR
	rrf	FSR
	movlw	low CURRENT_X1
	addwf	FSR
	
	; New value + AntiTwitchWidth < Current value ?
	movlw	AntiTwitchWidth + 1
	addwf	ADRESH,w
	subwf	INDF,w
	jnc	ADConversion_2
	movlw	AntiTwitchWidth		; Then change current value to new value + AntiTwitchWidth
	addwf	ADRESH,w
	movwf	INDF
	goto	ADConversion_End
	
ADConversion_2
	; New value - AntiTwitchWidth > Current value ?
	movlw	AntiTwitchWidth
	subwf	ADRESH,w
	subwf	INDF,w
	jc	ADConversion_End
	movlw	AntiTwitchWidth		; Then change current value to new value - AntiTwitchWidth
	subwf	ADRESH,w
	movwf	INDF
	
ADConversion_End
	movfw	INDF		; Return new current value in ADRESH
	movwf	ADRESH
	return
	
	
;-----------------------------------------------------------------------------------------


; Perform A/D conversion (with no logic to stop flickering)
; Takes about 25us in total
ADConversionRaw
	wait_us_M D'20'		; Wait 20us for A/D acquisition
	
	bsf	ADCON0,GO_DONE	; Start converstion
	btfsc	ADCON0,GO_DONE	; Loop until converstion complete
	goto	$-1
	
	movfw	ADRESH		; Invert A/D result
	sublw	0xFF
	movwf	ADRESH
	
	return


;-----------------------------------------------------------------------------------------

; Call from main loop
; RAM bank 0
;
; Controller id in MTEMP1 as follows:
; 0 - X1
; 1 - Y1
; 2 - Z1
; 3 - X2
; 4 - Y2
; 5 - Z2
; 6 - DATA
;
; Uses info in OFFSET_n, CTRL_MODE, CC_n_POS/NEG, MIN_n_POS/NEG, MAX_n_POS/NEG, LAST_n_POS/NEG
; Changes MTEMP2, MTEMP3, AARGB0, AARGB1, AARGB2, BARGB0, FSR
;
; Reads from A/D channel and sends appropriate MIDI message(s)
ADConversionToMIDI
	movlw	b'11000111'	; Clear analog input select bits
	andwf	ADCON0

	bcf	STATUS,C	; Set analog input select bits
	rlf	MTEMP1,w
	movwf	MTEMP2
	rlf	MTEMP2
	rlf	MTEMP2,w
	iorwf	ADCON0
	
	call	ADConversion	; Perform A/D conversion
	
	
	movlw	6		; DATA pot?
	subwf	MTEMP1,w
	jz	ADConversionToMIDI_DataPot
	
	
	movlw	low OFFSET_X1	; Get offset for joystick
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	
	subwf	ADRESH,w	; Convert A/D result to signed
	movwf	BARGB0
	
	; Adjustments to get 'perfect' centre point of joystick
	btfsc	BARGB0,7	; byte -ve ?
	goto	Byte_Negative
; byte +ve
	movfw	BARGB0
	sz			; If not 0, byte --
	decf	BARGB0
	sz			; If not 0, byte -- again
	decf	BARGB0
	goto	Byte_Adjusted
Byte_Negative
	incf	BARGB0		; byte ++
	sz			; If not 0, byte ++ again
	incf	BARGB0

Byte_Adjusted
	; Scale up factor for joystick: 0x01.40
	movlw	0x01
	movwf	AARGB0
	movlw	0x40
	movwf	AARGB1
	
	goto	ADConversionToMIDI_ScaleUpToByte
	
	
ADConversionToMIDI_DataPot
	movlw	0x80
	subwf	ADRESH,w	; Convert A/D result to signed
	movwf	BARGB0

	; Scale factor for DATA pot: 0x01.20
	movlw	0x01
	movwf	AARGB0
	movlw	0x20
	movwf	AARGB1


ADConversionToMIDI_ScaleUpToByte
	; Scale up factor * signed result -> AARGB0, AARGB1. AARGB2
	bsf	PCLATH,3
	call	FXM1608S
	bcf	PCLATH,3
	
	
	; Now cap result to make sure it fits into 8 bits
	
	btfss	AARGB0,0	; Positive result?
	goto	Result_Pos
	
; Negative result
	btfsc	AARGB1,7	; Result >= 0x80 ?
	goto	Result_Ok
	
	movlw	0x80		; Cap result at 0x80
	movwf	AARGB1
	goto	Result_Ok
	
Result_Pos
	btfss	AARGB1,7	; Result <= 0x7F ?
	goto	Result_Ok
	
	movlw	0x7F		; Cap result at 0x7F
	movwf	AARGB1
	
Result_Ok
	; Actual 8 bit signed value now in AARGB1
	
	
	; Check if simple mode or advanced mode
	movlw	b'00000001'
	movwf	MTEMP3
	bcf	STATUS,C
	
	movfw	MTEMP1		; Get bitmask corresponding to controller
	jz	$+5
	movwf	MTEMP2
	rlf	MTEMP3
	decfsz	MTEMP2
	goto	$-2
	
	movfw	MTEMP3
	bsf	STATUS,RP0	; RAM bank 1
	andwf	CTRL_MODE,w
	bcf	STATUS,RP0	; RAM bank 0
	
	jnz	AdvancedMode	; If non-zero result, then advanced mode (dual CC)
	
	
; Simple Mode
	bcf	FLAGS,ADVANCED_MODE
	
	; Convert to 7 bit unsigned -> AARGB0
	movlw	0x80
	addwf	AARGB1,w
	movwf	AARGB0
	bcf	STATUS,C
	rrf	AARGB0
	
SimpleMode_7bit
	movlw	low MAX_X1_POS	; Get Max value to MTEMP2
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP2
	
	movlw	low MIN_X1_POS	; Get Min value to MTEMP3
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP3
	
	; Max - Min -> BARGB0
	movfw	MTEMP3
	subwf	MTEMP2,w
	movwf	BARGB0
	
	; Value * (Max-Min) -> AARGB0, AARGB1
	bsf	PCLATH,3
	call	FXM0808S
	bcf	PCLATH,3
	
	; AARGB0, AARGB1 / 127 -> AARGB0, AARGB1
	movlw	D'127'
	movwf	BARGB0
	bsf	PCLATH,3
	call	FXD1608S
	bcf	PCLATH,3
	
	; AARGB1 + Min value -> AARGB1
	movfw	MTEMP3
	addwf	AARGB1
	
	; Indicate POS
	clrf	MTEMP2
	
	; Get CC -> MTEMP3
	movlw	low CC_X1_POS
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP3
	
	goto	SendMIDIMessageIfChanged
	

AdvancedMode
	bsf	FLAGS,ADVANCED_MODE
	
	movfw	AARGB1		; Copy 8 bit signed value to AARGB0
	movwf	AARGB0
	
	jz	AdvancedMode_Zero	; If value=0, this is special case
	
	btfss	AARGB0,7	; If positive, treat as simple mode
	goto	SimpleMode_7bit
	
	movlw	0x7F		; Convert to 7 bit unsigned
	addwf	AARGB0
	btfsc	AARGB0,7	; If -1, set to 0
	clrf	AARGB0
	
	
	movlw	low MAX_X1_NEG	; Get Max value to MTEMP2
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP2
	
	movlw	low MIN_X1_NEG	; Get Min value to MTEMP3
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP3
	
	; Max - Min -> BARGB0
	subwf	MTEMP2,w
	movwf	BARGB0
	
	; Value * (Max-Min) -> AARGB0, AARGB1
	bsf	PCLATH,3
	call	FXM0808S
	bcf	PCLATH,3
	
	; AARGB0, AARGB1 / 127 -> AARGB0, AARGB1
	movlw	D'127'
	movwf	BARGB0
	bsf	PCLATH,3
	call	FXD1608S
	bcf	PCLATH,3
	
	; AARGB1 + Min value -> AARGB1
	movfw	MTEMP3
	addwf	AARGB1
	
	; Indicate NEG
	movlw	1
	movwf	MTEMP2
	
	; Get CC -> MTEMP3
	movlw	low CC_X1_NEG
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP3
	
	goto	SendMIDIMessageIfChanged
	
	
AdvancedMode_Zero
	; Special case - need to send both POS and NEG controller messages
	
	; Send Max of NEG
	
	movlw	low MAX_X1_NEG
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	AARGB1
	
	movlw	1
	movwf	MTEMP2
	
	movlw	low CC_X1_NEG
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP3
	
	call	SendMIDIMessageIfChanged
	
	; Send Min of POS
	
	movlw	low MIN_X1_POS
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	AARGB1
	
	clrf	MTEMP2
	
	movlw	low CC_X1_POS
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP3
	
	goto	SendMIDIMessageIfChanged


;-----------------------------------------------------------------------------------------
	
	
; Call from ADConversionToMIDI (from main loop)
; RAM bank 0
;
; Controller id in MTEMP1 as follows:
; 0 - X1
; 1 - Y1
; 2 - Z1
; 3 - X2
; 4 - Y2
; 5 - Z2
; 6 - DATA
; 
; AARGB1 - scaled byte (CC value)
; MTEMP2 - 0=POS,1=NEG
; MTEMP3 - MIDI CC number
;
; Uses info in LAST_n_POS/NEG
SendMIDIMessageIfChanged

	btfss	FLAGS2,SUPPRESS_DATA_KNOB_CHANGE	; If SUPPRESS_DATA_KNOB_CHANGE flag set then ..
	goto	SendMIDIMessageIfChanged_1
	movlw	6					; check if DATA pot and ..
	subwf	MTEMP1,w
	jnz	SendMIDIMessageIfChanged_1
	bcf	FLAGS2,SUPPRESS_DATA_KNOB_CHANGE	; if so then clear SUPPRESS_DATA_KNOB_CHANGE flag and ..
	bsf	FLAGS2,DO_SUPPRESS_NOW			; set DO_SUPPRESS_NOW flag


SendMIDIMessageIfChanged_1
	; POS or NEG CC ?
	btfsc	MTEMP2,0
	goto	NEG_CC

; POS CC
	; Get last value -> MTEMP2
	movlw	low LAST_X1_POS
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP2
	
	; Compare CC value to last value
	movfw	AARGB1
	subwf	MTEMP2,w
	jz	SendMIDIMessageIfChanged_End	; If value hasn't changed skip the rest
	
	movlw	low LAST_X1_POS		; Save CC value to last value
	addwf	MTEMP1,w
	movwf	FSR
	movfw	AARGB1
	movwf	INDF
	
	btfss	FLAGS2,DO_SUPPRESS_NOW		; Don't send MIDI message if DO_SUPPRESS_NOW flag set
	goto	SendMIDIMessage
	goto	SendMIDIMessageIfChanged_End
	
NEG_CC
	; Get last value -> MTEMP2
	movlw	low LAST_X1_NEG
	addwf	MTEMP1,w
	movwf	FSR
	movfw	INDF
	movwf	MTEMP2
	
	; Compare CC value to last value
	movfw	AARGB1
	subwf	MTEMP2,w
	jz	SendMIDIMessageIfChanged_End	; If value hasn't changed skip the rest
	
	movlw	low LAST_X1_NEG		; Save CC value to last value
	addwf	MTEMP1,w
	movwf	FSR
	movfw	AARGB1
	movwf	INDF
	
	btfss	FLAGS2,DO_SUPPRESS_NOW		; Don't send MIDI message if DO_SUPPRESS_NOW flag set
	goto	SendMIDIMessage
	
SendMIDIMessageIfChanged_End
	bcf	FLAGS2,DO_SUPPRESS_NOW		; Clear DO_SUPPRESS_NOW flag
	return


;-----------------------------------------------------------------------------------------


; Call from main loop
; RAM bank 0
;
; MTEMP1 - Min data value
; MTEMP2 - Max data value
; MTEMP2 > MTEMP1 !!!
;
; Changes AARGB0, AARGB1, AARGB2, BARGB0, FSR
;
; Reads data pot A/D channel and scales result in range from MTEMP1 to MTEMP2
;
; Returns data value in AARGB1
GetDataPotValue
	bsf	ADCON0,CHS2	; Select analog input AN6 (DATA)
	bsf	ADCON0,CHS1
	bcf	ADCON0,CHS0
	
	incf	LAST_EDIT_VALUE,w	; If LAST_EDIT_VALUE is 0xFF ..
	jnz	GetDataPotValue_1
	
	call	ADConversionRaw		; Store raw A/D value to CURRENT_DATA
	movfw	ADRESH			; (prevents 1st value jumping immediately to data pot position)
	movwf	CURRENT_DATA
	
GetDataPotValue_1
	call	ADConversion	; Perform A/D conversion
	
	movlw	0x80
	subwf	ADRESH,w	; Convert A/D result to signed
	movwf	BARGB0

	; Scale factor for DATA pot: 0x01.20
	movlw	0x01
	movwf	AARGB0
	movlw	0x20
	movwf	AARGB1
	
	; Scale up factor * signed result -> AARGB0, AARGB1. AARGB2
	bsf	PCLATH,3
	call	FXM1608S
	bcf	PCLATH,3
	
	
	; Now cap result to make sure it fits into 8 bits
	
	btfss	AARGB0,0	; Positive result?
	goto	DataResult_Pos
	
; Negative result
	btfsc	AARGB1,7	; Result >= 0x80 ?
	goto	DataResult_Ok
	
	movlw	0x80		; Cap result at 0x80
	movwf	AARGB1
	clrf	AARGB2
	goto	DataResult_Ok
	
DataResult_Pos
	btfss	AARGB1,7	; Result <= 0x7F ?
	goto	DataResult_Ok
	
	movlw	0x7F		; Cap result at 0x7F
	movwf	AARGB1
	clrf	AARGB2
	
DataResult_Ok
	; Actual 8.8 bit signed value now in AARGB1.AARGB2
	
	; Convert to 8.8 bit unsigned -> AARGB0.AARGB1
	movlw	0x80
	addwf	AARGB1,w
	movwf	AARGB0
	movfw	AARGB2
	movwf	AARGB1
	
	; Max - Min -> BARGB0
	movfw	MTEMP1
	subwf	MTEMP2,w
	movwf	BARGB0
	
	; Value * (Max-Min) -> AARGB0, AARGB1. AARGB2
	bsf	PCLATH,3
	call	FXM1608U
	bcf	PCLATH,3
	
	; AARGB0, AARGB1. AARGB2 / 255 -> AARGB0, AARGB1. AARGB2
	clrf	BARGB0
	movlw	D'255'
	movwf	BARGB1
	bsf	PCLATH,3
	call	FXD2416U
	bcf	PCLATH,3
	
	; AARGB1 + Min value -> AARGB1
	movfw	MTEMP1
	addwf	AARGB1
	
	return


;-----------------------------------------------------------------------------------------


; Call with number in AARGB1
; Writes number to LCD
DisplayNumber
	; Convert result to BCD
	movfw	AARGB1
	call	ConvertToBcd
	
	; Write to result to LED
	movfw	AARGB0
	call	GetLEDNumber
	snz			; If 0, digit off
	movlw	0xFF
	movwf	LED_DIGIT1
	
	movfw	AARGB1
	call	GetLEDNumber
	jnz	DisplayNumber_1	; If not 0, jump to display digit
	movf	AARGB0		; If digit1 also 0, digit off
	snz
	movlw	0xFF
DisplayNumber_1
	movwf	LED_DIGIT2
	
	movfw	AARGB2
	call	GetLEDNumber
	movwf	LED_DIGIT3

	btfss	INTCON,TMR0IE		; If LED updates already off, don't change them here
	goto	DisplayNumber_2

	bcf	INTCON,TMR0IE		; LED updates off
	clrf	LED_INTERRUPT_COUNT	; Turn on display for 1.7s
	incf	LED_INTERRUPT_COUNT
	bsf	INTCON,TMR0IE		; LED updates on
	
	return
	
DisplayNumber_2
	clrf	LED_INTERRUPT_COUNT	; Turn on display for 1.7s
	incf	LED_INTERRUPT_COUNT
	return


;-----------------------------------------------------------------------------------------


; Call with byte in w
;
; Returns with:
; hundreds in AARGB0
; tens     in AARGB1
; ones     in AARGB2
ConvertToBcd
	movwf	AARGB2
	BinaryToBcd
	movfw	AARGB1
	andlw	b'00001111'
	movwf	AARGB2
	movlw	b'11110000'
	andwf	AARGB1
	bcf	STATUS,C
	rrf	AARGB1
	rrf	AARGB1
	rrf	AARGB1
	rrf	AARGB1
	return


;-----------------------------------------------------------------------------------------


; Writes referenced text onto LED	
; Call with offset text relative to LEDText_Base in MTEMP1
; Call only during main loop (or initialisation)
; RAM bank 0
WriteTextToLED
	movfw	MTEMP1
	call	GetLEDNumber
	movwf	LED_DIGIT1
	incf	MTEMP1
	movfw	MTEMP1
	call	GetLEDNumber
	movwf	LED_DIGIT2
	incf	MTEMP1,w
	call	GetLEDNumber
	movwf	LED_DIGIT3
	return
	
	
;-----------------------------------------------------------------------------------------


; Call from SendMIDIMessageIfChanged (from main loop)
; RAM bank 0
;
; Controller id in MTEMP1 as follows:
; 0 - X1
; 1 - Y1
; 2 - Z1
; 3 - X2
; 4 - Y2
; 5 - Z2
; 6 - DATA
;
; MTEMP3 - MIDI CC number
; AARGB1 - scaled byte (CC value)
;
; Sends MIDI message
SendMIDIMessage

	; Return if during on-delay period
	btfss	FLAGS,ON_DELAY_OVER
	return
	
	
	; If DATA pot then display value on LED
	movlw	6
	subwf	MTEMP1,w
	jnz	SendMIDIMessage_1
	
	movfw	AARGB1		; Save AARGB1
	movwf	BARGB1
	; Call with number in AARGB1
	; Writes number to LCD
	call	DisplayNumber
	movfw	BARGB1		; Restore AARGB1
	movwf	AARGB1
	
	
SendMIDIMessage_1
	; Pitch bend?
	movlw	D'128'
	subwf	MTEMP3,w
	jz	SendMIDIMessage_PitchBend
	
	; Aftertouch?
	movlw	D'129'
	subwf	MTEMP3,w
	jz	SendMIDIMessage_Aftertouch
	
	; Program change?
	movlw	D'130'
	subwf	MTEMP3,w
	jz	SendMIDIMessage_ProgChange
	
	; Off?
	movlw	D'131'
	subwf	MTEMP3,w
	rz
	
	
; CC
	movfw	MIDI_CHANNEL		; Combine message code with MIDI channel
	iorlw	0xB0
	movwf	TXREG			; Transmit
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0
	
	movfw	MTEMP3			; Transmit CC number
	movwf	TXREG
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0

	movfw	AARGB1			; Transmit CC value
	movwf	TXREG
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0
	
	goto	SendMIDIMessage_End
	
	
SendMIDIMessage_PitchBend
	movfw	MIDI_CHANNEL		; Combine message code with MIDI channel
	iorlw	0xE0
	movwf	TXREG			; Transmit
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0
	
	movlw	0			; Transmit LSB
	movwf	TXREG
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0

	movfw	AARGB1			; Transmit MSB
	movwf	TXREG
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0
	
	goto	SendMIDIMessage_End
	
	
SendMIDIMessage_Aftertouch
	movfw	MIDI_CHANNEL		; Combine message code with MIDI channel
	iorlw	0xD0
	movwf	TXREG			; Transmit
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0

	movfw	AARGB1			; Transmit value
	movwf	TXREG
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0
	
	goto	SendMIDIMessage_End
	
	
SendMIDIMessage_ProgChange
	movfw	MIDI_CHANNEL		; Combine message code with MIDI channel
	iorlw	0xC0
	movwf	TXREG			; Transmit
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0

	movfw	AARGB1			; Transmit value
	movwf	TXREG
	
	; Wait for byte to be transmitted
	bsf	STATUS,RP0	; RAM bank 1
	btfss	TXSTA,TRMT
	goto	$-1
	bcf	STATUS,RP0	; RAM bank 0

	
SendMIDIMessage_End
	; Now pause for 1ms to ensure that data rate will not exceed 50% of MIDI bandwidth
	wait_ms_M 1
	
	return
	
	
;-----------------------------------------------------------------------------------------

	
; Loads memory location from EEPROM pointed to by MEMORY_LOCATION (and writes MEMORY_LOCATION to EEPROM)
; Call only during main loop
; RAM bank 0
LoadMemoryLocation
	; Write current MEMORY_LOCATION to EEPROM
	movfw	MEMORY_LOCATION
	movwf	MTEMP2
	movlw	EE_MEMORY_LOCATION
	movwf	MTEMP1
	call	WriteByteToEEPROM_Main
	
; Alternative entry point for LoadMemoryLocation subroutine (doesn't write MEMORY_LOCATION to EEPROM)
; Called only during initialisation
; RAM bank 0
LoadMemoryLocation_Init
	; MEMORY_LOCATION * EE_MEM_WIDTH - EE_MEM1_START -> AARGB0, AARGB1
	movlw	EE_MEM_WIDTH
	movwf	AARGB0
	movfw	MEMORY_LOCATION
	movwf	BARGB0
	bsf	PCLATH,3
	call	FXM0808U
	bcf	PCLATH,3

	movlw	EE_MEM1_START
	subwf	AARGB1,f
	
	; EEPROM memory location start address in AARGB1
	
	; Check if this memory location is valid (read EE_CTRL_MODE)
	movfw	AARGB1
	movwf	MTEMP1
	call	ReadByteFromEEPROM_Main
	addlw	1		; EE_CTRL_MODE == 0xFF ?
	jnz	LoadMemoryLocation_Valid
	
	
	; Invalid memory location, so write default values to EEPROM
	clrf	MTEMP2			; Set EE_CTRL_MODE = 0
	call	WriteByteToEEPROM_Main
	incf	MTEMP1
	
	
	movlw	D'14'		; Count for 14 params (CC_*)
	movwf	MTEMP3
	
	movlw	D'131'		; Write 131 (off) to these 7 params
	movwf	MTEMP2
	
LoadMemoryLocation_Loop1
	call	WriteByteToEEPROM_Main
	incf	MTEMP1
	
	decfsz	MTEMP3
	goto	LoadMemoryLocation_Loop1
	
	movlw	D'14'		; Count for 14 params (MIN_*)
	movwf	MTEMP3
	
	clrf	MTEMP2		; Write 0 to these 15 params
	
LoadMemoryLocation_Loop2
	call	WriteByteToEEPROM_Main
	incf	MTEMP1
	
	decfsz	MTEMP3
	goto	LoadMemoryLocation_Loop2
	
	movlw	D'14'		; Count for 14 params (MAX_*)
	movwf	MTEMP3
	
	movlw	0x7F		; Write 0x7F to these 14 params
	movwf	MTEMP2
	
LoadMemoryLocation_Loop3
	call	WriteByteToEEPROM_Main
	incf	MTEMP1
	
	decfsz	MTEMP3
	goto	LoadMemoryLocation_Loop3
	
	
	movfw	AARGB1		; Reset MTEMP1 to point to base of memory location
	movwf	MTEMP1
	
	
LoadMemoryLocation_Valid
	; Load data from memory location
	
	movlw	low CTRL_MODE	; Point to CTRL_MODE
	movwf	FSR
	
	movlw	D'43'		; Count for 43 params (CTRL_MODE & CC_* & MIN_* & MAX_*)
	movwf	MTEMP3
	
LoadMemoryLocation_Loop4
	call	ReadByteFromEEPROM_Main

	movwf	INDF		; Write to RAM
	incf	FSR
	
	incf	MTEMP1
	decfsz	MTEMP3
	goto	LoadMemoryLocation_Loop4
	
	
	bsf	FLAGS2,SUPPRESS_DATA_KNOB_CHANGE	; Make sure data knob doesn't trigger change even if in new position
	return


;-----------------------------------------------------------------------------------------

	
; Call during main loop (or initialisation) to read byte from EEPROM
; 
; On call:
; --------
; RAM bank 0 selected
; MTEMP1 contains address location to read from
; 
; On return:
; ----------
; RAM bank 0 selected
; w contains byte read
ReadByteFromEEPROM_Main
	bcf	FLAGS2,INTERRUPTS_OFF
	btfss	INTCON,GIE
	bsf	FLAGS2,INTERRUPTS_OFF

	bsf	STATUS,RP1		; Bank 2
	bcf	INTCON,GIE		; Disable interrupts
	movfw	MTEMP1			; Address to read
	movwf	EEADR
	bsf	STATUS,RP0		; Bank 3
	bsf	EECON1,RD		; Start read operation
	bcf	STATUS,RP0		; Bank 2
	movfw	EEDATA			; Read byte
	bcf	STATUS,RP1		; Bank 0
	btfss	FLAGS2,INTERRUPTS_OFF
	bsf	INTCON,GIE		; Re-enable interrupts
	return
	

; Call during interrupt (or initialisation) to read byte from EEPROM
; 
; On call:
; --------
; RAM bank 0 selected
; TEMP1 contains address location to read from
; 
; On return:
; ----------
; RAM bank 0 selected
; w contains byte read
ReadByteFromEEPROM_Int
	bsf	STATUS,RP1		; Bank 2
	movfw	TEMP1			; Address to read
	movwf	EEADR
	bsf	STATUS,RP0		; Bank 3
	bsf	EECON1,RD		; Start read operation
	bcf	STATUS,RP0		; Bank 2
	movfw	EEDATA			; Read byte
	bcf	STATUS,RP1		; Bank 0
	return
	

; Call during main loop (or initialisation) to write byte to EEPROM
; 
; On call:
; --------
; RAM bank 0 selected
; MTEMP1 contains address location to write to
; MTEMP2 contains byte to write
; 
; On return:
; ----------
; RAM bank 0 selected
WriteByteToEEPROM_Main
	bcf	FLAGS2,INTERRUPTS_OFF
	btfss	INTCON,GIE
	bsf	FLAGS2,INTERRUPTS_OFF
	
	bsf	STATUS,RP1		; Bank 2
	
	bcf	INTCON,GIE		; Disable interrupts
	
	movfw	MTEMP1			; Address to write
	movwf	EEADR
	movfw	MTEMP2			; Data to write
	movwf	EEDATA
	
	bsf	STATUS,RP0		; Bank 3
	
	bsf	EECON1,WREN		; Enable writes
	
	movlw	0x55			; Special write sequence
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR
	
	bcf	EECON1,WREN		; Disable writes
	
	btfsc	EECON1,WR		; Wait for write to complete
	goto	$-1
	
	bcf	STATUS,RP0		; Bank 0
	bcf	STATUS,RP1
	
	btfss	FLAGS2,INTERRUPTS_OFF
	bsf	INTCON,GIE		; Re-enable interrupts
	return
	
	

; Call during interrupt (or initialisation) to write byte to EEPROM
; 
; On call:
; --------
; RAM bank 0 selected
; TEMP1 contains address location to write to
; TEMP2 contains byte to write
; 
; On return:
; ----------
; RAM bank 0 selected
WriteByteToEEPROM_Int
	bsf	STATUS,RP1		; Bank 2
	
	movfw	TEMP1			; Address to write
	movwf	EEADR
	movfw	TEMP2			; Data to write
	movwf	EEDATA
	
	bsf	STATUS,RP0		; Bank 3
	
	bsf	EECON1,WREN		; Enable writes
	
	movlw	0x55			; Special write sequence
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR
	
	bcf	EECON1,WREN		; Disable writes
	
	btfsc	EECON1,WR		; Wait for write to complete
	goto	$-1
	
	bcf	STATUS,RP0		; Bank 0
	bcf	STATUS,RP1
	return
	
	
;-----------------------------------------------------------------------------------------







;***********************************************************************


; Called every 6.5536ms
; Updates 1 digit of LED per call
UpdateLEDs
	bcf	INTCON,TMR0IF	; Reset TMR0 flag
	
	
	btfss	FLAGS,EDIT_PRESSED	; EDIT button held?
	goto	UpdateLEDs_1
	
	decf16	CALIBRATE_COUNT_L	; Decrement counter for edit button held
	
	movfw	CALIBRATE_COUNT_H	; Test if calibrate counter reached 0?
	iorwf	CALIBRATE_COUNT_L,w
	jnz	UpdateLEDs_1
	
	bcf	FLAGS,EDIT_PRESSED	; Clear EDIT button held flag (so nothing happens on release)
	bsf	FLAGS,CALIBRATE_OFFSETS	; Set flag to calibrate offsets in main loop
	
	
UpdateLEDs_1
	movfw	LED_INTERRUPT_COUNT	; Return if LED display is turned off
	rz
	
	incf	LED_INTERRUPT_COUNT	; Check if LED display needs to be turned off
	jz	UpdateLEDs_TurnOff
	
	; Return if during on-delay period
	btfss	FLAGS,ON_DELAY_OVER
	return
	
	; Check which digit to display
	btfsc	FLAGS,CURRENT_DIGIT_2
	goto	UpdateLEDs_Digit2
	
	btfsc	FLAGS,CURRENT_DIGIT_3
	goto	UpdateLEDs_Digit3
	
UpdateLEDs_Digit1
	bcf	PORTC,DIGIT3	; Turn off digit 3
	movfw	LED_DIGIT1	; Load digit 1 value into PORTD
	movwf	PORTD
	bsf	PORTC,DIGIT1	; Turn on digit 1
	
	bsf	FLAGS,CURRENT_DIGIT_2	; Set digit to update next time
	return
	
UpdateLEDs_Digit2
	bcf	PORTC,DIGIT1	; Turn off digit 1
	movfw	LED_DIGIT2	; Load digit 2 value into PORTD
	movwf	PORTD
	bsf	PORTC,DIGIT2	; Turn on digit 2
	
	bcf	FLAGS,CURRENT_DIGIT_2	; Set digit to update next time
	bsf	FLAGS,CURRENT_DIGIT_3
	return
	
UpdateLEDs_Digit3
	bcf	PORTC,DIGIT2	; Turn off digit 2
	movfw	LED_DIGIT3	; Load digit 3 value into PORTD
	movwf	PORTD
	bsf	PORTC,DIGIT3	; Turn on digit 3
	
	bcf	FLAGS,CURRENT_DIGIT_3	; Set digit to update next time
	return
	
UpdateLEDs_TurnOff
	movlw	b'11111000'	; Turn off all digits
	andwf	PORTC
	
	bsf	FLAGS,ON_DELAY_OVER	; Indicate that on-delay is over
	
	return
	
	
;***********************************************************************
	
	
; RAM bank 0
MIDIByteReceived
	movfw	RCSTA			; Read and save and error code
	movwf	TEMP2
	
	movfw	RCREG			; Read MIDI byte
	movwf	TEMP1			; Save in TEMP1
	
	btfsc	TEMP2,OERR		; Check for overrun error
	goto	MIDIByteReceived_Error
	btfsc	TEMP2,FERR		; Check for framing error
	goto	MIDIByteReceived_Error
	
	
	btfss	TEMP1,7			; Header or data byte?
	goto	MIDIByteReceived_DataByte
	
; header byte
	bcf	FLAGS,MIDI_CC_MSG	; Clear CC message flag
	
	movlw	b'11110000'		; Mask out channel
	andwf	TEMP1
	
	movlw	0xB0			; CC message?
	subwf	TEMP1,w
	jz	MIDIByteReceived_CC
	
	movlw	0xC0			; Program change message?
	subwf	TEMP1,w
	jz	MIDIByteReceived_ProgChange
	
	movlw	0xD0			; Aftertouch message?
	subwf	TEMP1,w
	jz	MIDIByteReceived_Aftertouch
	
	movlw	0xE0			; Pitch bend message?
	subwf	TEMP1,w
	jz	MIDIByteReceived_PitchBend
	
	return				; Ignore everything else
	

MIDIByteReceived_DataByte
	btfss	FLAGS,MIDI_CC_MSG	; If not expected MIDI CC value then learn value
	goto	Learn_CC_Value
	
	bcf	FLAGS,MIDI_CC_MSG	; Clear CC message flag
	
	movfw	TEMP1
	goto	Learn_CC
	

MIDIByteReceived_CC
	bsf	FLAGS,MIDI_CC_MSG	; Set CC message flag
	return
	
MIDIByteReceived_ProgChange
	movlw	D'130'
	goto	Learn_CC
	
MIDIByteReceived_Aftertouch
	movlw	D'129'
	goto	Learn_CC
	
MIDIByteReceived_PitchBend
	movlw	D'128'
	goto	Learn_CC
	
	
MIDIByteReceived_Error
	bcf	RCSTA,CREN		; Reset receive logic / clear error flag
	bsf	RCSTA,CREN
	return
	
	
Learn_CC
	movwf	TEMP1			; Save controller
	
	movlw	4			; If in edit modes 4 or 7 then record CC
	subwf	EDIT_MODE_STAGE,w
	jz	Learn_CC_1
	movlw	7
	subwf	EDIT_MODE_STAGE,w
	rnz
	
Learn_CC_1
	movfw	TEMP1			; Copy controller to CC_LEARN
	movwf	CC_LEARN
	return
	
Learn_CC_Value
	movlw	5			; If in edit modes 5, 6, 8 or 9 then record value
	subwf	EDIT_MODE_STAGE,w
	jz	Learn_CC_Value_1
	movlw	6
	subwf	EDIT_MODE_STAGE,w
	jz	Learn_CC_Value_1
	movlw	8
	subwf	EDIT_MODE_STAGE,w
	jz	Learn_CC_Value_1
	movlw	9
	subwf	EDIT_MODE_STAGE,w
	rnz
	
Learn_CC_Value_1
	movfw	TEMP1			; Copy value to CC_VALUE_LEARN
	movwf	CC_VALUE_LEARN
	return
	
;-----------------------------------------------------------------------------------------


; Lookup tables for LED digits

TABLE_START	EQU	$
	org	$ + 0xFF - ($ % 0x100)

GetLEDNumber
	addwf	PCL
LEDText_Base
LEDNumbers
	dt	b'11000000'	; 0
	dt	b'11111001'	; 1
	dt	b'10100100'	; 2
	dt	b'10110000'	; 3
	dt	b'10011001'	; 4
	dt	b'10010010'	; 5
	dt	b'10000010'	; 6
	dt	b'11011000'	; 7
	dt	b'10000000'	; 8
	dt	b'10010000'	; 9
	
LEDText_CAL
	dt	b'11000110'	; 'CAL'
	dt	b'10001000'
	dt	b'11000111'
	
LEDText_X1
	dt	b'10001001'	; 'H1 '
	dt	b'11111001'
	dt	b'11111111'
	
LEDText_Y1
	dt	b'10010001'	; 'y1 '
	dt	b'11111001'
	dt	b'11111111'
	
LEDText_Z1
	dt	b'10100100'	; 'Z1 '
	dt	b'11111001'
	dt	b'11111111'
	
LEDText_X2
	dt	b'10001001'	; 'H2 '
	dt	b'10100100'
	dt	b'11111111'
	
LEDText_Y2
	dt	b'10010001'	; 'y2 '
	dt	b'10100100'
	dt	b'11111111'
	
LEDText_Z2
	dt	b'10100100'	; 'Z2 '
	dt	b'10100100'
	dt	b'11111111'
	
LEDText_DATA
	dt	b'11000000'	; 'DAt'
	dt	b'10001000'
	dt	b'10000111'
	
LEDText_EDIT
	dt	b'10000110'	; 'Edt'
	dt	b'10100001'
	dt	b'10000111'
	
LEDText_PitchBend
	dt	b'10001100'	; 'Pch'
	dt	b'10100111'
	dt	b'10001011'
	
LEDText_Aftertouch
	dt	b'10001000'	; 'Aft'
	dt	b'10001110'
	dt	b'10000111'
	
LEDText_ProgChange
	dt	b'10001100'	; 'PCh'
	dt	b'11000110'
	dt	b'10001011'
	
LEDText_Off
	dt	b'11000000'	; 'Off'
	dt	b'10001110'
	dt	b'10001110'
	
LEDText_Simple
	dt	b'10111111'	; '-S-'
	dt	b'10010010'
	dt	b'10111111'
	
LEDText_Advanced
	dt	b'10111111'	; '-A-'
	dt	b'10001000'
	dt	b'10111111'
	
	
	
;***********************************************************************
	
	
ButtonsChange
	movlw	0x3C		; Allow 20ms to avoid switch bounce
	movwf	TMR1H
	movlw	0xB0
	movwf	TMR1L
	
	bsf	T1CON,TMR1ON	; Start Timer1

	movfw	PORTB		; Read PORTB to end mismatch condition
	bcf	INTCON,RBIF	; Clear interrupt flag and return
	return
	
	
;***********************************************************************
	
	
ReadButtons
	bcf	T1CON,TMR1ON	; Stop Timer1
	bcf	PIR1,TMR1IF	; Clear interrupt flag
	
	
	btfss	PORTB,EDIT		; EDIT button pressed?
	goto	ReadButtons_EDIT
	
	btfsc	FLAGS,EDIT_PRESSED	; EDIT button just released?
	goto	ReadButtons_EDIT_Released
	
	btfss	PORTB,MEMORY		; MEMORY button pressed?
	goto	ReadButtons_MEMORY
	bcf	FLAGS,MEMORY_PRESSED	; Clear flag
	
	return
	
	
ReadButtons_EDIT
	bsf	FLAGS,EDIT_PRESSED	; Set flag
	
	movlw	3			; Set calibrate counter to count 0x300 * 6.5536ms (~5 seconds)
	movwf	CALIBRATE_COUNT_H
	clrf	CALIBRATE_COUNT_L

	return
	
ReadButtons_EDIT_Released
	bsf	FLAGS,ON_DELAY_OVER	; Indicate that on-delay is over (in case button is pressed very soon after power on)
	bcf	FLAGS,EDIT_PRESSED	; Clear flag
	bsf	FLAGS2,RESET_LAST_EDIT_VALUE	; Indicate LAST_EDIT_VALUE needs to be reset
	
	; If EDIT_MODE_STAGE == 2, jump to stage 11
	movlw	2
	subwf	EDIT_MODE_STAGE,w
	jz	ReadButtons_EDIT_Released_Stage11
	
	incf	EDIT_MODE_STAGE		; Move to next edit mode stage
	
	; If EDIT_MODE_STAGE == 7, test simple/advanced mode
	movlw	7
	subwf	EDIT_MODE_STAGE,w
	jz	ReadButtons_EDIT_Released_TestMode
	
	; If EDIT_MODE_STAGE == 2, check EDIT_MODE_CTRL_SELECTED ..
	; If no contoller has been selected enter MIDI channel select stage
	; Otherwise enter Simple/Advanced mode select stage
	movlw	2
	subwf	EDIT_MODE_STAGE,w
	rnz
	movfw	EDIT_MODE_CTRL_SELECTED
	sz
	incf	EDIT_MODE_STAGE
	
	return
	
ReadButtons_EDIT_Released_Stage11
	movlw	D'11'			; Jump to stage 11
	movwf	EDIT_MODE_STAGE
	return
	
ReadButtons_EDIT_Released_TestMode
	movfw	EDIT_MODE_CTRL_MODE	; If in advanced mode, stage 7 is correct so return
	rnz
	
	movlw	D'10'			; Otherwise jump to stage 10
	movwf	EDIT_MODE_STAGE
	return

	
ReadButtons_MEMORY
	bsf	FLAGS,ON_DELAY_OVER	; Indicate that on-delay is over (in case button is pressed very soon after power on)
	
	clrf	LED_INTERRUPT_COUNT	; Turn off display
	decf	LED_INTERRUPT_COUNT
	
	movfw	EDIT_MODE_STAGE		; If not in edit mode ..
	snz
	bsf	FLAGS,MEMORY_PRESSED	; .. set flag
	
	clrf	EDIT_MODE_CTRL_SELECTED
	clrf	EDIT_MODE_STAGE		; Exit edit mode
	bsf	FLAGS2,SUPPRESS_DATA_KNOB_CHANGE	; Make sure data knob doesn't trigger change even if in new position
	
	return
	
	
;***********************************************************************


; Displays number in w on LED
; Changes contents of AARGB0, AARGB1, AARGB2
DebugDisplayNumber
	movwf	AARGB1
	call	DisplayNumber
	movlw	0xFF
	movwf	AARGB2
	movlw	0xFF
	movwf	AARGB1
	movlw	D'50'
	movwf	AARGB0
	decfsz	AARGB0
	goto	$-1
	decfsz	AARGB1
	goto	$-5
	decfsz	AARGB2
	goto	$-9
	return
	
	
;***********************************************************************

; Failsafe check that table doesn't overun 256 byte block
	org	TABLE_START + 0xFF - (TABLE_START % 0x100) + 0x100
	nop
	
;***********************************************************************

;#######################################################################


	; Program bank 1
	
	org	0x0800
	
; Maths Functions
#include <maths.inc>
	
	
;#######################################################################


	; Program bank 2
	
	org	0x1000
	
	nop
	
	
;#######################################################################


	; Program bank 3
	
	org	0x1800
	
	nop


;#######################################################################


	; EEPROM
	
 	org	0x2100

; 	de	0xFF		; Set EE_MEMORY_LOCATION to 'invalid'


;#######################################################################


	END			 ; directive 'end of program'


